/*
 * Copyright (C) 2015-2017 Voipac.
 *
 * Configuration settings for the Fedevel i.MX6 REX board.
 *
 * SPDX-License-Identifier:    GPL-2.0+
 */

#ifndef __MX6REX_COMMON_CONFIG_H
#define __MX6REX_COMMON_CONFIG_H

#define CONFIG_MX6

#ifdef CONFIG_MX6S
#define CONFIG_MX6DL
#define CONFIG_SYS_NOSMP "nosmp"
#else
#define CONFIG_SYS_NOSMP
#endif

/* uncomment for SECURE mode support */
/* #define CONFIG_SECURE_BOOT */

#ifdef CONFIG_SECURE_BOOT
#ifndef CONFIG_CSF_SIZE
#define CONFIG_CSF_SIZE 0x4000
#endif
#endif

#include "mx6_common.h"
#include <linux/sizes.h>

#define CONFIG_DISPLAY_CPUINFO
#define CONFIG_DISPLAY_BOARDINFO

#include <asm/arch/imx-regs.h>
#include <asm/imx-common/gpio.h>

/* TAGs */
#define CONFIG_CMDLINE_TAG
#define CONFIG_SETUP_MEMORY_TAGS
#define CONFIG_INITRD_TAG
#define CONFIG_REVISION_TAG

#define CONFIG_SYS_GENERIC_BOARD

/* Size of malloc() pool */
#define CONFIG_SYS_MALLOC_LEN               (16 * SZ_1M)

#define CONFIG_BOARD_EARLY_INIT_F
#define CONFIG_BOARD_LATE_INIT

/* Gpio Configs */
#define CONFIG_MXC_GPIO

/* Uart Configs */
#define CONFIG_MXC_UART
#define CONFIG_MXC_UART_BASE                UART1_BASE

/* Fuse Configs */
#define CONFIG_CMD_FUSE
#ifdef CONFIG_CMD_FUSE
#define CONFIG_MXC_OCOTP
#endif

/* MMC Configs */
#define CONFIG_FSL_ESDHC
#define CONFIG_FSL_USDHC

#define CONFIG_MMC
#define CONFIG_CMD_MMC
#define CONFIG_GENERIC_MMC
#define CONFIG_BOUNCE_BUFFER
#define CONFIG_CMD_EXT2
#define CONFIG_CMD_EXT4
#define CONFIG_CMD_EXT4_WRITE
#define CONFIG_CMD_FAT
#define CONFIG_DOS_PARTITION

/* SPI Configs */
#define CONFIG_CMD_SF
#ifdef CONFIG_CMD_SF
#define CONFIG_SPI_FLASH
#define CONFIG_SPI_FLASH_SST
#define CONFIG_MXC_SPI
#define CONFIG_SF_DEFAULT_SPEED             20000000
#define CONFIG_SF_DEFAULT_MODE              SPI_MODE_0
#endif

/* Ethernet config */
#define CONFIG_CMD_PING
#define CONFIG_CMD_DHCP
#define CONFIG_CMD_MII
#define CONFIG_CMD_NET
#define CONFIG_FEC_MXC
#define CONFIG_MII
#define IMX_FEC_BASE                        ENET_BASE_ADDR
#define CONFIG_FEC_XCV_TYPE                 RGMII
#define CONFIG_ETHPRIME                     "FEC"
#define CONFIG_FEC_MXC_PHYADDR              3

#define CONFIG_PHYLIB
#define CONFIG_PHY_MICREL
#define CONFIG_PHY_MICREL_KSZ9021

#define CONFIG_ARP_TIMEOUT                  200UL

/* allow to overwrite serial and ethaddr */
#define CONFIG_ENV_OVERWRITE
#define CONFIG_CONS_INDEX                   1
#define CONFIG_BAUDRATE                     115200

/* Command definition */
#include <config_cmd_default.h>

/* Miscellaneous commands */
#define CONFIG_CMD_BMODE
#define CONFIG_CMD_BOOTZ
#define CONFIG_CMD_SETEXPR
#undef CONFIG_CMD_IMLS

#define CONFIG_BOOTDELAY                    1

#define CONFIG_LOADADDR                     0x10800000
#define CONFIG_SYS_TEXT_BASE                0x17800000
#define CONFIG_SYS_MMC_IMG_LOAD_PART        1

#define CONFIG_OF_LIBFDT

#ifndef CONFIG_SYS_DCACHE_OFF
#define CONFIG_CMD_CACHE
#endif

/* I2C Configs */
#define CONFIG_CMD_I2C
#define CONFIG_SYS_I2C
#define CONFIG_SYS_I2C_MXC
#define CONFIG_SYS_I2C_SPEED                100000

/* Framebuffer */
#if 0
#define CONFIG_VIDEO
#define CONFIG_VIDEO_IPUV3
#define CONFIG_CFB_CONSOLE
#define CONFIG_VGA_AS_SINGLE_DEVICE
#define CONFIG_SYS_CONSOLE_IS_IN_ENV
#define CONFIG_SYS_CONSOLE_OVERWRITE_ROUTINE
#define CONFIG_VIDEO_BMP_RLE8
#define CONFIG_SPLASH_SCREEN
#define CONFIG_SPLASH_SCREEN_ALIGN
#define CONFIG_BMP_16BPP
#define CONFIG_VIDEO_LOGO
#define CONFIG_VIDEO_BMP_LOGO
#ifdef CONFIG_MX6DL
#define CONFIG_IPUV3_CLK 198000000
#else
#define CONFIG_IPUV3_CLK 264000000
#endif
#define CONFIG_IMX_HDMI
#define CONFIG_IMX_VIDEO_SKIP
#endif

/* Sata Configs */
#if defined(CONFIG_MX6Q) || defined(CONFIG_MX6QP)
#define CONFIG_CMD_SATA
#endif
#ifdef CONFIG_CMD_SATA
#define CONFIG_DWC_AHSATA
#define CONFIG_SYS_SATA_MAX_DEVICE          1
#define CONFIG_DWC_AHSATA_PORT_ID           0
#define CONFIG_DWC_AHSATA_BASE_ADDR         SATA_ARB_BASE_ADDR
#define CONFIG_LBA48
#define CONFIG_LIBATA
#endif

/* PCI Configs */
/*
 * imx6 q/dl/solo pcie would be failed to work properly in kernel, if
 * the pcie module is initiated/enumerated both in uboot and linux
 * kernel.
 * rootcause:imx6 q/dl/solo pcie don't have the reset mechanism.
 * it is only be RESET by the POR. So, the pcie module only be
 * initialized/enumerated once in one POR.
 * Set to use pcie in kernel defaultly, mask the pcie config here.
 * Remove the mask freely, if the uboot pcie functions, rather than
 * the kernel's, are required.
 */
#undef CONFIG_CMD_PCI
#ifdef CONFIG_CMD_PCI
#define CONFIG_PCI
#define CONFIG_PCI_PNP
#define CONFIG_PCI_SCAN_SHOW
#define CONFIG_PCIE_IMX
#define CONFIG_PCIE_IMX_PERST_GPIO          IMX_GPIO_NR(5, 2)
#endif

/* USB Configs */
#define CONFIG_CMD_USB
#ifdef CONFIG_CMD_USB
#define CONFIG_USB_EHCI
#define CONFIG_USB_EHCI_MX6
#define CONFIG_USB_STORAGE
#define CONFIG_EHCI_HCD_INIT_AFTER_RESET
#define CONFIG_USB_HOST_ETHER
#define CONFIG_USB_ETHER_ASIX
#define CONFIG_MXC_USB_PORTSC               (PORT_PTS_UTMI | PORT_PTS_PTW)
#define CONFIG_MXC_USB_FLAGS                0
#define CONFIG_USB_MAX_CONTROLLER_COUNT     1 /* Enabled USB controller number */
#endif

/* Miscellaneous configurable options */
#define CONFIG_SYS_LONGHELP
#define CONFIG_SYS_HUSH_PARSER
#define CONFIG_AUTO_COMPLETE
#define CONFIG_SYS_CBSIZE                   1024

/* Print Buffer Size */
#define CONFIG_SYS_MAXARGS                  256
#define CONFIG_SYS_BARGSIZE                 CONFIG_SYS_CBSIZE

#define CONFIG_CMD_MEMTEST
#define CONFIG_SYS_MEMTEST_START            (MMDC0_ARB_BASE_ADDR)
#define CONFIG_SYS_MEMTEST_END              (CONFIG_SYS_MEMTEST_START + SZ_1M)
#define CONFIG_SYS_MEMTEST_SCRATCH          CONFIG_LOADADDR

#define CONFIG_SYS_LOAD_ADDR                CONFIG_LOADADDR

#define CONFIG_CMDLINE_EDITING
#define CONFIG_STACKSIZE                    SZ_128K

/* Physical Memory Map */
#define CONFIG_NR_DRAM_BANKS                1
#define PHYS_SDRAM                          MMDC0_ARB_BASE_ADDR

#define CONFIG_SYS_SDRAM_BASE               PHYS_SDRAM
#define CONFIG_SYS_INIT_RAM_ADDR            IRAM_BASE_ADDR
#define CONFIG_SYS_INIT_RAM_SIZE            IRAM_SIZE

#define CONFIG_SYS_INIT_SP_OFFSET \
	(CONFIG_SYS_INIT_RAM_SIZE - GENERATED_GBL_DATA_SIZE)
#define CONFIG_SYS_INIT_SP_ADDR \
	(CONFIG_SYS_INIT_RAM_ADDR + CONFIG_SYS_INIT_SP_OFFSET)

#if defined(CONFIG_ANDROID_SUPPORT)
#include "mx6rexandroid_common.h"
#endif

#endif                         /* __MX6REX_COMMON_CONFIG_H */
